* Program: formutil.sas;
* written by David Yao;
/* 
     This program assigns value labels to most character variables in the 2009 AHS
     metro and national datasets.  
*/

options nocenter nosource;

*********************  Update Section   *******************************************************;

* Update the following for proper path *****/;

%let patha=C:\AHS09\National\New100611; /* put in the proper path to your data here */

***********************************************************************************************;

%let year = 09;  /* current survey year */

libname ahs&year. "&patha.";

filename outfile1 "&patha.\fmtincl.txt" lrecl=32767;

options fmtsearch=(ahs&year.); /* use this library for format catalog */

%macro loop1 (indd= );

* indd = Dataset name;

/* write contents listing to a dataset */
proc contents data=ahs&year..&indd. out=&indd. noprint;
run;

/* convert all variable names to uppercase */

DATA &indd.;  
     SET &indd.;

 Name = UPCASE(Name);

RUN;


/* sort by name and type of the variable, keeping only character variables and the numeric SMSA */

proc sort data=&indd. ; by name type;

 where type = 2       /* char type */ 
    or name = 'SMSA'; /* SMSA is numeric in AHS 2007 and 2009 national surveys */

run;

Data newformat&year.; set ahs&year..newformat&year.;

type = 2;

if &year. = 09 and name = 'SMSA' 
   and newfmt = 'SMSA' /* SMSA is numeric in AHS 2007 and 2009 national surveys */ 

then type = 1;

run; 


/* Sort format database by variable name */
proc sort data=newformat&year.; 
by name type; 
run;

/* merge format list with variable name list */
data v&indd. (keep=name newfmt type);
  merge newformat&year. (in=a) &indd. (in=b) ;
  by name type;
  if a and b;
run;

/* create empty text file (this clears the file from previous runs of the macro)*/
data _null_; file outfile1; run;


/* write format statements to text file */
data _null_ ;  file outfile1 mod;
set v&indd.;
put 'FORMAT ' name  $ newfmt $ +(-1) '. ; ' ;
run;

/* Assign the formats to the AHS dataset */
proc datasets ddname=ahs&year.;
 modify &indd.;
 %include outfile1; /* inserts format statements created in previous step */
run;

%mend loop1;

/* These lines modify the individual AHS datasets.  comment them out if you use
     only the flattened file. */
%loop1(indd=person);
%loop1(indd=owner) ;
%loop1(indd=ratiov) ; 
%loop1(indd=mortg) ;
%loop1(indd=homimp) ;
%loop1(indd=rmov) ;
%loop1(indd=jtw) ;
%loop1(indd=newhouse) ;

/* This line modifies the flattened file.  Comment it out if you don't use it. */
%loop1(indd=AHS2009);

quit;




